@@ -1,47 +1,77 @@
+from logging import Logger
 from typing import Callable, List, Optional
 
 import urwid
 from deltachat import Chat, const
 from emoji import demojize
 
-from ..event import ChatListMonitor
 from ..scli import LazyEvalListWalker, ListBoxPlus
 from ..util import is_pinned, shorten_text
 
 
-class ListItem(urwid.Button):
-    def __init__(self, caption: list, callback: Callable, arg=None) -> None:
+class ChatListItem(urwid.Button):
+    def __init__(self, caption: list, callback: Callable, chat: Chat) -> None:
         super().__init__("")
-        urwid.connect_signal(self, "click", callback, arg)
+        urwid.connect_signal(self, "click", callback, chat)
         self._w = urwid.AttrMap(
-            urwid.SelectableIcon(caption, 3), None, focus_map="status_bar"
+            urwid.SelectableIcon(caption, 3), None, focus_map="focused_item"
         )
 
 
-class ChatListWidget(ListBoxPlus, ChatListMonitor):
-    def __init__(
-        self,
-        keymap: dict,
-        select_chat_callback: Callable,
-        display_emoji: bool,
-    ) -> None:  # noqa
+class ChatListWidget(ListBoxPlus):
+    signals = ["chat_selected"]
+
+    def __init__(self, keymap: dict, display_emoji: bool, logger: Logger) -> None:
         self.keymap = keymap
-        self._select_chat = select_chat_callback
-        self.updating = False
         self.display_emoji = display_emoji
-        self.current_chat_id = None
-        self.chats: List[Chat] = []
+        self.selected_chat: Optional[Chat] = None
+        self.logger = logger
         super().__init__(
             LazyEvalListWalker(urwid.MonitoredList(), self._get_chat_widget, 0)
         )
 
-    def chatlist_changed(
-        self, current_chat_index: Optional[int], chats: List[Chat]
-    ) -> None:
-        self._update(current_chat_index, chats)
+    def _item_clicked(self, _button, chat: Chat) -> None:
+        self.select_chat(chat)
 
-    def chat_selected(self, index: Optional[int], chats: List[Chat]) -> None:
-        self._update(index, chats)
+    def _get_chat(self, position) -> Optional[Chat]:
+        if 0 <= position < len(self.contents):
+            return self.contents[position]
+        return None
+
+    def set_chats(self, chats: List[Chat]) -> None:
+        prev_position = self.get_focus()[1]
+        self.contents = chats
+        if prev_position is not None:
+            self.try_set_focus(prev_position)
+        if self.selected_chat and self.selected_chat not in self.contents:
+            self.select_chat(None)
+
+    def select_chat(self, chat: Chat) -> None:
+        self.logger.debug("Chat selected: %s", chat)
+        self.selected_chat = chat
+        urwid.emit_signal(self, "chat_selected", chat)
+
+    def select_next_chat(self) -> None:
+        if self.selected_chat:
+            i = self.contents.index(self.selected_chat)
+            if i < 0:  # no chat selected, skip
+                return
+            i -= 1
+            if i < 0:
+                i = len(self.contents) - 1
+            self.try_set_focus(i)
+            self.select_chat(self.contents[i])
+
+    def select_previous_chat(self) -> None:
+        if self.selected_chat:
+            i = self.contents.index(self.selected_chat)
+            if i < 0:  # no chat selected, skip
+                return
+            i += 1
+            if i >= len(self.contents):
+                i = 0
+            self.try_set_focus(i)
+            self.select_chat(self.contents[i])
 
     def keypress(self, size: list, key: str) -> Optional[str]:
         key = super().keypress(size, key)
@@ -53,27 +83,6 @@ class ChatListWidget(ListBoxPlus, ChatListMonitor):
             return key
         return None
 
-    def _update(self, current_chat_index: Optional[int], chats: List[Chat]) -> None:
-        if self.updating:
-            return
-        self.updating = True
-
-        if current_chat_index is None:
-            self.current_chat_id = None
-        else:
-            self.current_chat_id = chats[current_chat_index].id
-        self.chats = chats
-        self.contents = chats
-        if current_chat_index is not None:
-            self.try_set_focus(current_chat_index)
-
-        self.updating = False
-
-    def _get_chat(self, position) -> Optional[Chat]:
-        if 0 <= position < len(self.chats):
-            return self.chats[position]
-        return None
-
     def _get_chat_widget(self, chat: Chat, position: int) -> urwid.Widget:
         elements: list = []
 
@@ -92,7 +101,7 @@ class ChatListWidget(ListBoxPlus, ChatListMonitor):
 
         name = shorten_text(chat.get_name(), 40)
         label = f"{name if self.display_emoji else demojize(name)}"
-        if chat.id == self.current_chat_id:
+        if self.select_chat and chat == self.selected_chat:
             label = ("cur_chat", label)  # type: ignore
         elements.append(label)
 
@@ -104,7 +113,7 @@ class ChatListWidget(ListBoxPlus, ChatListMonitor):
             else:
                 elements.append(("unread_chat", f"({new_messages})"))
 
-        widget = ListItem(elements, self._chat_change, position)
+        widget = ChatListItem(elements, self._item_clicked, chat)
 
         if is_pinned(chat):
             next_chat = self._get_chat(position + 1)
@@ -113,6 +122,3 @@ class ChatListWidget(ListBoxPlus, ChatListMonitor):
                 widget = urwid.Pile([widget, divider])
 
         return widget
-
-    def _chat_change(self, _button, index: int) -> None:
-        self._select_chat(index)
